@@ -21,6 +21,8 @@ from relstorage.adapters.batch import MySQLRowBatcher
 from relstorage.adapters.batch import OracleRowBatcher
 from relstorage.adapters.batch import PostgreSQLRowBatcher
 from zope.interface import implements
+import os
+import sys
 
 try:
     from hashlib import md5
@@ -54,13 +56,16 @@ class ObjectMover(object):
         'replace_temp',
         'move_from_temp',
         'update_current',
-        )
+        'download_blob',
+        'upload_blob',
+    )
 
-    def __init__(self, database_name, keep_history, runner=None,
+    def __init__(self, database_name, options, runner=None,
             Binary=None, inputsizes=None, version_detector=None):
         # The inputsizes parameter is for Oracle only.
         self.database_name = database_name
-        self.keep_history = keep_history
+        self.keep_history = options.keep_history
+        self.blob_chunk_size = options.blob_chunk_size
         self.runner = runner
         self.Binary = Binary
         self.inputsizes = inputsizes
@@ -373,7 +378,7 @@ class ObjectMover(object):
 
 
     def postgresql_on_store_opened(self, cursor, restart=False):
-        """Create the temporary table for storing objects"""
+        """Create the temporary tables for storing objects"""
         # note that the md5 column is not used if self.keep_history == False.
         stmt = """
         CREATE TEMPORARY TABLE temp_store (
@@ -382,29 +387,55 @@ class ObjectMover(object):
             md5         CHAR(32),
             state       BYTEA
         ) ON COMMIT DROP;
-        CREATE UNIQUE INDEX temp_store_zoid ON temp_store (zoid)
+        CREATE UNIQUE INDEX temp_store_zoid ON temp_store (zoid);
+
+        CREATE TEMPORARY TABLE temp_blob_chunk (
+            zoid        BIGINT NOT NULL,
+            chunk_num   BIGINT NOT NULL,
+            chunk       OID
+        ) ON COMMIT DROP;
+        CREATE UNIQUE INDEX temp_blob_chunk_key
+            ON temp_blob_chunk (zoid, chunk_num);
+
+        -- This trigger removes blobs that get replaced before being
+        -- moved to blob_chunk.  Note that it is never called when
+        -- the temp_blob_chunk table is being dropped or truncated.
+        CREATE TRIGGER temp_blob_chunk_delete 
+            BEFORE DELETE ON temp_blob_chunk
+            FOR EACH ROW
+            EXECUTE PROCEDURE temp_blob_chunk_delete_trigger();
         """
         cursor.execute(stmt)
 
     def mysql_on_store_opened(self, cursor, restart=False):
         """Create the temporary table for storing objects"""
         if restart:
-            stmt = """
-            DROP TEMPORARY TABLE IF EXISTS temp_store
-            """
+            stmt = "DROP TEMPORARY TABLE IF EXISTS temp_store"
+            cursor.execute(stmt)
+            stmt = "DROP TEMPORARY TABLE IF EXISTS temp_blob_chunk"
             cursor.execute(stmt)
 
         # note that the md5 column is not used if self.keep_history == False.
         stmt = """
         CREATE TEMPORARY TABLE temp_store (
-            zoid        BIGINT NOT NULL PRIMARY KEY,
-            prev_tid    BIGINT NOT NULL,
+            zoid        BIGINT UNSIGNED NOT NULL PRIMARY KEY,
+            prev_tid    BIGINT UNSIGNED NOT NULL,
             md5         CHAR(32),
             state       LONGBLOB
         ) ENGINE MyISAM
         """
         cursor.execute(stmt)
 
+        stmt = """
+        CREATE TEMPORARY TABLE temp_blob_chunk (
+            zoid        BIGINT UNSIGNED NOT NULL,
+            chunk_num   BIGINT UNSIGNED NOT NULL,
+                        PRIMARY KEY (zoid, chunk_num),
+            chunk       LONGBLOB
+        ) ENGINE MyISAM
+        """
+        cursor.execute(stmt)
+
     # no store connection initialization needed for Oracle
     oracle_on_store_opened = None
 
@@ -436,7 +467,7 @@ class ObjectMover(object):
             (oid, prev_tid, md5sum, encodestring(data)),
             rowkey=oid,
             size=len(data),
-            )
+        )
 
     def mysql_store_temp(self, cursor, batcher, oid, prev_tid, data):
         """Store an object in the temporary table."""
@@ -451,7 +482,7 @@ class ObjectMover(object):
             rowkey=oid,
             size=len(data),
             command='REPLACE',
-            )
+        )
 
     def oracle_store_temp(self, cursor, batcher, oid, prev_tid, data):
         """Store an object in the temporary table."""
@@ -460,7 +491,8 @@ class ObjectMover(object):
         else:
             md5sum = None
 
-        if len(data) <= 2000:
+        size = len(data)
+        if size <= 2000:
             # Send data inline for speed.  Oracle docs say maximum size
             # of a RAW is 2000 bytes.
             stmt = "BEGIN relstorage_op.store_temp(:1, :2, :3, :4); END;"
@@ -469,8 +501,8 @@ class ObjectMover(object):
                 'oid prev_tid md5sum rawdata',
                 (oid, prev_tid, md5sum, data),
                 rowkey=oid,
-                size=len(data),
-                )
+                size=size,
+            )
         else:
             # Send data as a BLOB
             row = {
@@ -478,14 +510,14 @@ class ObjectMover(object):
                 'prev_tid': prev_tid,
                 'md5sum': md5sum,
                 'blobdata': data,
-                }
+            }
             batcher.insert_into(
                 "temp_store (zoid, prev_tid, md5, state)",
                 ":oid, :prev_tid, :md5sum, :blobdata",
                 row,
                 rowkey=oid,
-                size=len(data),
-                )
+                size=size,
+            )
 
 
 
@@ -502,33 +534,35 @@ class ObjectMover(object):
 
         if data is not None:
             encoded = encodestring(data)
+            size = len(data)
         else:
             encoded = None
+            size = 0
 
         if self.keep_history:
             batcher.delete_from("object_state", zoid=oid, tid=tid)
             row_schema = """
                 %s, %s,
                 COALESCE((SELECT tid FROM current_object WHERE zoid = %s), 0),
-                %s, decode(%s, 'base64')
+                %s, %s, decode(%s, 'base64')
             """
             batcher.insert_into(
-                "object_state (zoid, tid, prev_tid, md5, state)",
+                "object_state (zoid, tid, prev_tid, md5, state_size, state)",
                 row_schema,
-                (oid, tid, oid, md5sum, encoded),
+                (oid, tid, oid, md5sum, size, encoded),
                 rowkey=(oid, tid),
-                size=len(data or ''),
-                )
+                size=size,
+            )
         else:
             batcher.delete_from('object_state', zoid=oid)
             if data:
                 batcher.insert_into(
-                    "object_state (zoid, tid, state)",
-                    "%s, %s, decode(%s, 'base64')",
-                    (oid, tid, encoded),
+                    "object_state (zoid, tid, state_size, state)",
+                    "%s, %s, %s, decode(%s, 'base64')",
+                    (oid, tid, size, encoded),
                     rowkey=oid,
-                    size=len(data),
-                    )
+                    size=size,
+                )
 
     def mysql_restore(self, cursor, batcher, oid, tid, data):
         """Store an object directly, without conflict detection.
@@ -542,33 +576,35 @@ class ObjectMover(object):
 
         if data is not None:
             encoded = self.Binary(data)
+            size = len(data)
         else:
             encoded = None
+            size = 0
 
         if self.keep_history:
             row_schema = """
                 %s, %s,
                 COALESCE((SELECT tid FROM current_object WHERE zoid = %s), 0),
-                %s, %s
+                %s, %s, %s
             """
             batcher.insert_into(
-                "object_state (zoid, tid, prev_tid, md5, state)",
+                "object_state (zoid, tid, prev_tid, md5, state_size, state)",
                 row_schema,
-                (oid, tid, oid, md5sum, encoded),
+                (oid, tid, oid, md5sum, size, encoded),
                 rowkey=(oid, tid),
-                size=len(data or ''),
+                size=size,
                 command='REPLACE',
-                )
+            )
         else:
             if data:
                 batcher.insert_into(
-                    "object_state (zoid, tid, state)",
-                    "%s, %s, %s",
-                    (oid, tid, encoded),
+                    "object_state (zoid, tid, state_size, state)",
+                    "%s, %s, %s, %s",
+                    (oid, tid, size, encoded),
                     rowkey=oid,
-                    size=len(data),
+                    size=size,
                     command='REPLACE',
-                    )
+                )
             else:
                 batcher.delete_from('object_state', zoid=oid)
 
@@ -582,7 +618,12 @@ class ObjectMover(object):
         else:
             md5sum = None
 
-        if not data or len(data) <= 2000:
+        if data is not None:
+            size = len(data)
+        else:
+            size = 0
+
+        if size <= 2000:
             # Send data inline for speed.  Oracle docs say maximum size
             # of a RAW is 2000 bytes.
             if self.keep_history:
@@ -592,8 +633,8 @@ class ObjectMover(object):
                     'oid tid md5sum rawdata',
                     (oid, tid, md5sum, data),
                     rowkey=(oid, tid),
-                    size=len(data or ''),
-                    )
+                    size=size,
+                )
             else:
                 stmt = "BEGIN relstorage_op.restore(:1, :2, :3); END;"
                 batcher.add_array_op(
@@ -601,8 +642,8 @@ class ObjectMover(object):
                     'oid tid rawdata',
                     (oid, tid, data),
                     rowkey=(oid, tid),
-                    size=len(data or ''),
-                    )
+                    size=size,
+                )
 
         else:
             # Send as a BLOB
@@ -611,32 +652,39 @@ class ObjectMover(object):
                     'oid': oid,
                     'tid': tid,
                     'md5sum': md5sum,
+                    'state_size': size,
                     'blobdata': data,
-                    }
+                }
                 row_schema = """
                     :oid, :tid,
                     COALESCE((SELECT tid
                               FROM current_object
                               WHERE zoid = :oid), 0),
-                    :md5sum, :blobdata
+                    :md5sum, :state_size, :blobdata
                 """
                 batcher.insert_into(
-                    "object_state (zoid, tid, prev_tid, md5, state)",
+                    "object_state (zoid, tid, prev_tid, md5, state_size, state)",
                     row_schema,
                     row,
                     rowkey=(oid, tid),
-                    size=len(data or ''),
-                    )
+                    size=size,
+                )
             else:
                 batcher.delete_from('object_state', zoid=oid)
                 if data:
+                    row = {
+                        'oid': oid,
+                        'tid': tid,
+                        'state_size': size,
+                        'blobdata': data,
+                    }
                     batcher.insert_into(
-                        "object_state (zoid, tid, state)",
-                        ":oid, :tid, :blobdata",
-                        {'oid': oid, 'tid': tid, 'blobdata': data},
+                        "object_state (zoid, tid, state_size, state)",
+                        ":oid, :tid, :state_size, :blobdata",
+                        row,
                         rowkey=oid,
-                        size=len(data),
-                        )
+                        size=size,
+                    )
 
 
 
@@ -789,7 +837,7 @@ class ObjectMover(object):
 
 
 
-    def generic_move_from_temp(self, cursor, tid):
+    def generic_move_from_temp(self, cursor, tid, txn_has_blobs):
         """Moved the temporarily stored objects to permanent storage.
 
         Returns the list of oids stored.
@@ -797,14 +845,18 @@ class ObjectMover(object):
         if self.keep_history:
             if self.database_name == 'oracle':
                 stmt = """
-                INSERT INTO object_state (zoid, tid, prev_tid, md5, state)
-                SELECT zoid, :1, prev_tid, md5, state
+                INSERT INTO object_state
+                    (zoid, tid, prev_tid, md5, state_size, state)
+                SELECT zoid, :1, prev_tid, md5,
+                    COALESCE(LENGTH(state), 0), state
                 FROM temp_store
                 """
             else:
                 stmt = """
-                INSERT INTO object_state (zoid, tid, prev_tid, md5, state)
-                SELECT zoid, %s, prev_tid, md5, state
+                INSERT INTO object_state
+                    (zoid, tid, prev_tid, md5, state_size, state)
+                SELECT zoid, %s, prev_tid, md5,
+                    COALESCE(LENGTH(state), 0), state
                 FROM temp_store
                 """
             cursor.execute(stmt, (tid,))
@@ -812,8 +864,8 @@ class ObjectMover(object):
         else:
             if self.database_name == 'mysql':
                 stmt = """
-                REPLACE INTO object_state (zoid, tid, state)
-                SELECT zoid, %s, state
+                REPLACE INTO object_state (zoid, tid, state_size, state)
+                SELECT zoid, %s, COALESCE(LENGTH(state), 0), state
                 FROM temp_store
                 """
                 cursor.execute(stmt, (tid,))
@@ -827,18 +879,40 @@ class ObjectMover(object):
 
                 if self.database_name == 'oracle':
                     stmt = """
-                    INSERT INTO object_state (zoid, tid, state)
-                    SELECT zoid, :1, state
+                    INSERT INTO object_state (zoid, tid, state_size, state)
+                    SELECT zoid, :1, COALESCE(LENGTH(state), 0), state
                     FROM temp_store
                     """
                 else:
                     stmt = """
-                    INSERT INTO object_state (zoid, tid, state)
-                    SELECT zoid, %s, state
+                    INSERT INTO object_state (zoid, tid, state_size, state)
+                    SELECT zoid, %s, COALESCE(LENGTH(state), 0), state
                     FROM temp_store
                     """
                 cursor.execute(stmt, (tid,))
 
+            if txn_has_blobs:
+                stmt = """
+                DELETE FROM blob_chunk
+                WHERE zoid IN (SELECT zoid FROM temp_store)
+                """
+                cursor.execute(stmt)
+
+        if txn_has_blobs:
+            if self.database_name == 'oracle':
+                stmt = """
+                INSERT INTO blob_chunk (zoid, tid, chunk_num, chunk)
+                SELECT zoid, :1, chunk_num, chunk
+                FROM temp_blob_chunk
+                """
+            else:
+                stmt = """
+                INSERT INTO blob_chunk (zoid, tid, chunk_num, chunk)
+                SELECT zoid, %s, chunk_num, chunk
+                FROM temp_blob_chunk
+                """
+            cursor.execute(stmt, (tid,))
+
         stmt = """
         SELECT zoid FROM temp_store
         """
@@ -923,3 +997,339 @@ class ObjectMover(object):
         """
         cursor.execute(stmt, (tid,))
 
+
+
+
+    def postgresql_download_blob(self, cursor, oid, tid, filename):
+        """Download a blob into a file."""
+        stmt = """
+        SELECT chunk_num, chunk
+        FROM blob_chunk
+        WHERE zoid = %s
+            AND tid = %s
+        ORDER BY chunk_num
+        """
+
+        f = None
+        bytes = 0
+
+        try:
+            cursor.execute(stmt, (oid, tid))
+            for chunk_num, loid in cursor.fetchall():
+                blob = cursor.connection.lobject(loid, 'rb')
+
+                if chunk_num == 0:
+                    # Use the native psycopg2 blob export functionality
+                    blob.export(filename)
+                    blob.close()
+                    bytes = os.path.getsize(filename)
+                    continue
+
+                if f is None:
+                    f = open(filename, 'ab') # Append, chunk 0 was an export
+                read_chunk_size = self.blob_chunk_size
+                reader = iter(lambda: blob.read(read_chunk_size), '')
+                for read_chunk in reader:
+                    f.write(read_chunk)
+                    bytes += len(read_chunk)
+        except:
+            if f is not None:
+                f.close()
+                os.remove(filename)
+            raise
+
+        if f is not None:
+            f.close()
+        return bytes
+
+    def mysql_download_blob(self, cursor, oid, tid, filename):
+        """Download a blob into a file."""
+        stmt = """
+        SELECT chunk
+        FROM blob_chunk
+        WHERE zoid = %s
+            AND tid = %s
+            AND chunk_num = %s
+        """
+
+        f = None
+        bytes = 0
+        try:
+            chunk_num = 0
+            while True:
+                cursor.execute(stmt, (oid, tid, chunk_num))
+                rows = list(cursor)
+                if rows:
+                    assert len(rows) == 1
+                    chunk = rows[0][0]
+                else:
+                    # No more chunks.  Note: if there are no chunks at
+                    # all, then this method should not write a file.
+                    break
+
+                if f is None:
+                    f = open(filename, 'wb')
+                f.write(chunk)
+                bytes += len(chunk)
+                chunk_num += 1
+        except:
+            if f is not None:
+                f.close()
+                os.remove(filename)
+            raise
+
+        if f is not None:
+            f.close()
+        return bytes
+
+    def oracle_download_blob(self, cursor, oid, tid, filename):
+        """Download a blob into a file."""
+        stmt = """
+        SELECT chunk
+        FROM blob_chunk
+        WHERE zoid = :1
+            AND tid = :2
+        ORDER BY chunk_num
+        """
+
+        f = None
+        bytes = 0
+        # Current versions of cx_Oracle only support offsets up
+        # to sys.maxint or 4GB, whichever comes first.
+        maxsize = min(sys.maxint, 1<<32)
+        try:
+            cursor.execute(stmt, (oid, tid))
+            while True:
+                try:
+                    blob, = cursor.fetchone()
+                except TypeError:
+                    # No more chunks.  Note: if there are no chunks at
+                    # all, then this method should not write a file.
+                    break 
+
+                if f is None:
+                    f = open(filename, 'wb')
+                # round off the chunk-size to be a multiple of the oracle
+                # blob chunk size to maximize performance
+                read_chunk_size = int(max(round(
+                    1.0 * self.blob_chunk_size / blob.getchunksize()), 1) * 
+                    blob.getchunksize())
+                offset = 1 # Oracle still uses 1-based indexing.
+                reader = iter(lambda: blob.read(offset, read_chunk_size), '')
+                for read_chunk in reader:
+                    f.write(read_chunk)
+                    bytes += len(read_chunk)
+                    offset += len(read_chunk)
+                    if offset > maxsize:
+                        # We have already read the maximum we can store
+                        # so we can assume we are done. If we do not break
+                        # off here, cx_Oracle will throw an overflow
+                        # exception anyway.
+                        break
+        except:
+            if f is not None:
+                f.close()
+                os.remove(filename)
+            raise
+
+        if f is not None:
+            f.close()
+        return bytes
+
+
+
+    def postgresql_upload_blob(self, cursor, oid, tid, filename):
+        """Upload a blob from a file.
+
+        If serial is None, upload to the temporary table.
+        """
+        if tid is not None:
+            if self.keep_history:
+                delete_stmt = """
+                DELETE FROM blob_chunk
+                WHERE zoid = %s AND tid = %s
+                """
+                cursor.execute(delete_stmt, (oid, tid))
+            else:
+                delete_stmt = "DELETE FROM blob_chunk WHERE zoid = %s"
+                cursor.execute(delete_stmt, (oid,))
+
+            use_tid = True
+            insert_stmt = """
+            INSERT INTO blob_chunk (zoid, tid, chunk_num, chunk)
+            VALUES (%(oid)s, %(tid)s, %(chunk_num)s, %(loid)s)
+            """
+
+        else:
+            use_tid = False
+            delete_stmt = "DELETE FROM temp_blob_chunk WHERE zoid = %s"
+            cursor.execute(delete_stmt, (oid,))
+
+            insert_stmt = """
+            INSERT INTO temp_blob_chunk (zoid, chunk_num, chunk)
+            VALUES (%(oid)s, %(chunk_num)s, %(loid)s)
+            """
+
+        blob = None
+        # PostgreSQL only supports up to 2GB of data per BLOB.
+        maxsize = 1<<31
+        filesize = os.path.getsize(filename)
+
+        if filesize <= maxsize:
+            # File is small enough to fit in one chunk, just use
+            # psycopg2 native file copy support
+            blob = cursor.connection.lobject(0, 'wb', 0, filename)
+            blob.close()
+            params = dict(oid=oid, chunk_num=0, loid=blob.oid)
+            if use_tid:
+                params['tid'] = tid
+            cursor.execute(insert_stmt, params)
+            return
+
+        # We need to divide this up into multiple chunks
+        f = open(filename, 'rb')
+        try:
+            chunk_num = 0
+            while True:
+                blob = cursor.connection.lobject(0, 'wb')
+                params = dict(oid=oid, chunk_num=chunk_num, loid=blob.oid)
+                if use_tid:
+                    params['tid'] = tid
+                cursor.execute(insert_stmt, params)
+
+                write_chunk_size = self.blob_chunk_size
+                for i in xrange(maxsize / write_chunk_size):
+                    write_chunk = f.read(write_chunk_size)
+                    if not blob.write(write_chunk):
+                        # EOF.
+                        return
+                if not blob.closed:
+                    blob.close()
+                chunk_num += 1
+        finally:
+            f.close()
+            if blob is not None and not blob.closed:
+                blob.close()
+
+    def mysql_upload_blob(self, cursor, oid, tid, filename):
+        """Upload a blob from a file.
+
+        If serial is None, upload to the temporary table.
+        """
+        if tid is not None:
+            if self.keep_history:
+                delete_stmt = """
+                DELETE FROM blob_chunk
+                WHERE zoid = %s AND tid = %s
+                """
+                cursor.execute(delete_stmt, (oid, tid))
+            else:
+                delete_stmt = "DELETE FROM blob_chunk WHERE zoid = %s"
+                cursor.execute(delete_stmt, (oid,))
+
+            use_tid = True
+            insert_stmt = """
+            INSERT INTO blob_chunk (zoid, tid, chunk_num, chunk)
+            VALUES (%s, %s, %s, %s)
+            """
+        else:
+            use_tid = False
+            delete_stmt = "DELETE FROM temp_blob_chunk WHERE zoid = %s"
+            cursor.execute(delete_stmt, (oid,))
+
+            insert_stmt = """
+            INSERT INTO temp_blob_chunk (zoid, chunk_num, chunk)
+            VALUES (%s, %s, %s)
+            """
+
+        f = open(filename, 'rb')
+        try:
+            chunk_num = 0
+            while True:
+                chunk = f.read(self.blob_chunk_size)
+                if not chunk and chunk_num > 0:
+                    # EOF.  Note that we always write at least one
+                    # chunk, even if the blob file is empty.
+                    break
+                if use_tid:
+                    params = (oid, tid, chunk_num, chunk)
+                else:
+                    params = (oid, chunk_num, chunk)
+                cursor.execute(insert_stmt, params)
+                chunk_num += 1
+        finally:
+            f.close()
+
+    def oracle_upload_blob(self, cursor, oid, tid, filename):
+        """Upload a blob from a file.
+
+        If serial is None, upload to the temporary table.
+        """
+        if tid is not None:
+            if self.keep_history:
+                delete_stmt = """
+                DELETE FROM blob_chunk
+                WHERE zoid = :1 AND tid = :2
+                """
+                cursor.execute(delete_stmt, (oid, tid))
+            else:
+                delete_stmt = "DELETE FROM blob_chunk WHERE zoid = :1"
+                cursor.execute(delete_stmt, (oid,))
+
+            use_tid = True
+            insert_stmt = """
+            INSERT INTO blob_chunk (zoid, tid, chunk_num, chunk)
+            VALUES (:oid, :tid, :chunk_num, empty_blob())
+            """
+            select_stmt = """
+            SELECT chunk FROM blob_chunk
+            WHERE zoid=:oid AND tid=:tid AND chunk_num=:chunk_num
+            """
+
+        else:
+            use_tid = False
+            delete_stmt = "DELETE FROM temp_blob_chunk WHERE zoid = :1"
+            cursor.execute(delete_stmt, (oid,))
+
+            insert_stmt = """
+            INSERT INTO temp_blob_chunk (zoid, chunk_num, chunk)
+            VALUES (:oid, :chunk_num, empty_blob())
+            """
+            select_stmt = """
+            SELECT chunk FROM temp_blob_chunk 
+            WHERE zoid=:oid AND chunk_num=:chunk_num
+            """
+
+        f = open(filename, 'rb')
+        # Current versions of cx_Oracle only support offsets up
+        # to sys.maxint or 4GB, whichever comes first. We divide up our
+        # upload into chunks within this limit.
+        maxsize = min(sys.maxint, 1<<32)
+        try:
+            chunk_num = 0
+            while True:
+                blob = None
+                params = dict(oid=oid, chunk_num=chunk_num)
+                if use_tid:
+                    params['tid'] = tid
+                cursor.execute(insert_stmt, params)
+                cursor.execute(select_stmt, params)
+                blob, = cursor.fetchone()
+                blob.open()
+                write_chunk_size = int(max(round(
+                    1.0 * self.blob_chunk_size / blob.getchunksize()), 1) * 
+                    blob.getchunksize())
+                offset = 1 # Oracle still uses 1-based indexing.
+                for i in xrange(maxsize / write_chunk_size):
+                    write_chunk = f.read(write_chunk_size)
+                    if not blob.write(write_chunk, offset):
+                        # EOF.
+                        return
+                    offset += len(write_chunk)
+                if blob is not None and blob.isopen():
+                    blob.close()
+                chunk_num += 1
+        finally:
+            f.close()
+            if blob is not None and blob.isopen():
+                blob.close()
