@@ -93,7 +93,7 @@ class OracleAdapter(object):
             )
         self.mover = ObjectMover(
             database_name='oracle',
-            keep_history=self.keep_history,
+            options=options,
             runner=self.runner,
             Binary=cx_Oracle.Binary,
             inputsizes={
@@ -102,6 +102,7 @@ class OracleAdapter(object):
                 'oid': cx_Oracle.NUMBER,
                 'tid': cx_Oracle.NUMBER,
                 'prev_tid': cx_Oracle.NUMBER,
+                'chunk_num': cx_Oracle.NUMBER,
                 'md5sum': cx_Oracle.STRING,
                 },
             )
@@ -127,20 +128,26 @@ class OracleAdapter(object):
 
         if self.keep_history:
             self.packundo = OracleHistoryPreservingPackUndo(
+                database_name='oracle',
                 connmanager=self.connmanager,
                 runner=self.runner,
                 locker=self.locker,
+                options=options,
                 )
             self.dbiter = HistoryPreservingDatabaseIterator(
+                database_name='oracle',
                 runner=self.runner,
                 )
         else:
             self.packundo = OracleHistoryFreePackUndo(
+                database_name='oracle',
                 connmanager=self.connmanager,
                 runner=self.runner,
                 locker=self.locker,
+                options=options,
                 )
             self.dbiter = HistoryFreeDatabaseIterator(
+                database_name='oracle',
                 runner=self.runner,
                 )
 
