@@ -22,16 +22,21 @@ from relstorage.tests.hptestbase import HistoryPreservingRelStorageTests
 from relstorage.tests.hptestbase import HistoryPreservingToFileStorage
 import logging
 import os
+import sys
 import unittest
 
+
+base_dbname = os.environ.get('RELSTORAGETEST_DBNAME', 'relstoragetest')
+
+
 class UseOracleAdapter:
     def make_adapter(self):
         from relstorage.adapters.oracle import OracleAdapter
         dsn = os.environ.get('ORACLE_TEST_DSN', 'XE')
         if self.keep_history:
-            db = 'relstoragetest'
+            db = base_dbname
         else:
-            db = 'relstoragetest_hf'
+            db = base_dbname + '_hf'
         return OracleAdapter(
             user=db,
             password='relstoragetest',
@@ -50,9 +55,9 @@ class ZConfigTests:
         os.close(fd)
         try:
             if self.keep_history:
-                dbname = 'relstoragetest'
+                dbname = base_dbname
             else:
-                dbname = 'relstoragetest_hf'
+                dbname = base_dbname + '_hf'
             conf = """
             %%import relstorage
             <zodb main>
@@ -61,6 +66,7 @@ class ZConfigTests:
                 read-only false
                 keep-history %s
                 replica-conf %s
+                blob-chunk-size 10MB
                 <oracle>
                   user %s
                   password relstoragetest
@@ -105,6 +111,7 @@ class ZConfigTests:
                 self.assertEqual(
                     adapter.connmanager.replica_selector.replica_conf,
                     replica_conf)
+                self.assertEqual(storage._options.blob_chunk_size, 10485760)
             finally:
                 db.close()
         finally:
@@ -132,10 +139,10 @@ class HFOracleFromFile(UseOracleAdapter, HistoryFreeFromFileStorage):
     pass
 
 db_names = {
-    'data': 'relstoragetest',
-    '1': 'relstoragetest',
-    '2': 'relstoragetest2',
-    'dest': 'relstoragetest2',
+    'data': base_dbname,
+    '1': base_dbname,
+    '2': base_dbname + '2',
+    'dest': base_dbname + '2',
     }
 
 def test_suite():
@@ -165,37 +172,58 @@ def test_suite():
     else:
         from relstorage.tests.blob.testblob import storage_reusable_suite
         dsn = os.environ.get('ORACLE_TEST_DSN', 'XE')
-        for keep_history in (False, True):
-            def create_storage(name, blob_dir, keep_history=keep_history):
-                from relstorage.storage import RelStorage
-                from relstorage.adapters.oracle import OracleAdapter
-                db = db_names[name]
-                if not keep_history:
-                    db += '_hf'
-                adapter = OracleAdapter(
-                    user=db,
-                    password='relstoragetest',
-                    dsn=dsn,
-                    options=Options(keep_history=keep_history),
+        from relstorage.tests.util import shared_blob_dir_choices
+        for shared_blob_dir in shared_blob_dir_choices:
+            for keep_history in (False, True):
+                def create_storage(name, blob_dir,
+                        shared_blob_dir=shared_blob_dir,
+                        keep_history=keep_history, **kw):
+                    from relstorage.storage import RelStorage
+                    from relstorage.adapters.oracle import OracleAdapter
+                    db = db_names[name]
+                    if not keep_history:
+                        db += '_hf'
+                    options = Options(
+                        keep_history=keep_history,
+                        shared_blob_dir=shared_blob_dir,
+                        blob_dir=os.path.abspath(blob_dir),
+                        **kw)
+                    adapter = OracleAdapter(
+                        user=db,
+                        password='relstoragetest',
+                        dsn=dsn,
+                        options=options,
                     )
-                storage = RelStorage(adapter, name=name, create=True,
-                    blob_dir=os.path.abspath(blob_dir))
-                storage.zap_all()
-                return storage
+                    storage = RelStorage(adapter, name=name, options=options)
+                    storage.zap_all()
+                    return storage
 
-            if keep_history:
-                prefix = 'HPOracle'
-                pack_test_name = 'blob_packing.txt'
-            else:
-                prefix = 'HFOracle'
-                pack_test_name = 'blob_packing_history_free.txt'
+                prefix = 'Oracle%s%s' % (
+                    (shared_blob_dir and 'Shared' or 'Unshared'),
+                    (keep_history and 'WithHistory' or 'NoHistory'),
+                )
 
-            suite.addTest(storage_reusable_suite(
-                prefix, create_storage,
-                test_blob_storage_recovery=True,
-                test_packing=True,
-                test_undo=keep_history,
-                pack_test_name=pack_test_name,
+                # If the blob directory is a cache, don't test packing,
+                # since packing can not remove blobs from all caches.
+                test_packing = shared_blob_dir
+
+                if keep_history:
+                    pack_test_name = 'blob_packing.txt'
+                else:
+                    pack_test_name = 'blob_packing_history_free.txt'
+
+                # cx_Oracle blob support can only address up to sys.maxint on
+                # 32-bit systems, 4GB otherwise.
+                blob_size = min(sys.maxint, 1<<32)
+
+                suite.addTest(storage_reusable_suite(
+                    prefix, create_storage,
+                    test_blob_storage_recovery=True,
+                    test_packing=test_packing,
+                    test_undo=keep_history,
+                    pack_test_name=pack_test_name,
+                    test_blob_cache=(not shared_blob_dir),
+                    large_blob_size=(not shared_blob_dir) and blob_size + 100,
                 ))
 
     return suite
